/*
 * wavefront.cpp
 *
 *  Created on: Jul 4, 2010
 *      Author: puel
 */

#include "wavefront.h"
#include <fstream>
#include <GL/gl.h>
#include <iostream>

using std::ifstream;
typedef std::vector<triangulo>::iterator triangulos;

Wavefront::Wavefront(GLuint textura, std::string arquivo, ListaDeFrames frames, Cor cor) : Objeto(frames, cor), textura(textura) {
	vertices.push_back(coordenada());
	normais.push_back(coordenada());
	faces.push_back(triangulo());
	texturas.push_back(uv());

	char opcao[2];
	ifstream aberto(arquivo.c_str(), ifstream::in);
	while( !aberto.eof() ) {
		opcao[0] = '#';
		aberto.get(opcao[0]).get(opcao[1]);
		if( opcao[0] == 'v' && opcao[1] == ' ' ) {
			coordenada nova;
			aberto >> nova.x;
			aberto >> nova.y;
			aberto >> nova.z;
			vertices.push_back(nova);
		}else if( opcao[0] == 'v' && opcao[1] == 'n' ) {
			coordenada nova;
			aberto >> nova.x;
			aberto >> nova.y;
			aberto >> nova.z;
			normais.push_back(nova);
		}else if( opcao[0] == 'v' && opcao[1] == 't') {
			uv novo;
			aberto >> novo.x;
			aberto >> novo.y;
			texturas.push_back(novo);
		}else if( opcao[0] == 'f' ) {
			triangulo novo;
			aberto >> novo.v1;
			aberto.get();
			aberto >> novo.t1;
			aberto.get();
			aberto >> novo.n1;

			aberto >> novo.v2;
			aberto.get();
			aberto >> novo.t2;
			aberto.get();
			aberto >> novo.n2;

			aberto >> novo.v3;
			aberto.get();
			aberto >> novo.t3;
			aberto.get();
			aberto >> novo.n3;

			faces.push_back(novo);
		}
		aberto.ignore(1024, '\n');
	}
}

Wavefront::~Wavefront() {
}

void Wavefront::_desenhar() {
	glPushMatrix();
	glScaled(3.0, 3.0, 3.0);

	glBindTexture(GL_TEXTURE_2D, textura);
	for(triangulos i = faces.begin(); i != faces.end(); ++i) {
		glBegin(GL_TRIANGLES);

		coordenada vertice = vertices[i->v1];
		coordenada normal = normais[i->n1];
		uv texAtual = texturas[i->t1];
		glTexCoord2d(texAtual.x, texAtual.y);
//		glTexCoord2d(0.0, 0.0);
		glVertex3d(vertice.x, vertice.y, vertice.z);
		glNormal3d(normal.x, normal.y, normal.z);

		vertice = vertices[i->v2];
		normal = normais[i->n2];
		texAtual = texturas[i->t2];
		glTexCoord2d(texAtual.x, texAtual.y);
//		glTexCoord2d(1.0, 0.0);
		glVertex3d(vertice.x, vertice.y, vertice.z);
		glNormal3d(normal.x, normal.y, normal.z);

		vertice = vertices[i->v3];
		normal = normais[i->n3];
		texAtual = texturas[i->t3];
		glTexCoord2d(texAtual.x, texAtual.y);
//		glTexCoord2d(1.0, 1.0);
		glVertex3d(vertice.x, vertice.y, vertice.z);
		glNormal3d(normal.x, normal.y, normal.z);

		glEnd();
	}
	glPopMatrix();
}
